@@ -509,6 +509,9 @@ static int remove_interface(struct supplicant_task *task)
 
 	DBG("task %p", task);
 
+	if (task->path == NULL)
+		return 0;
+
 #if 0
 	if (task->created == FALSE) {
 		connman_device_set_powered(task->device, FALSE);
@@ -648,7 +651,7 @@ static int remove_network(struct supplicant_task *task)
 
 	DBG("task %p", task);
 
-	if (task->netpath == NULL)
+	if (task->netpath == NULL || task->path == NULL)
 		return -EINVAL;
 
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
@@ -733,6 +736,9 @@ static int disconnect_network(struct supplicant_task *task)
 
 	DBG("task %p", task);
 
+	if (task->path == NULL)
+		return -EINVAL;
+
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
 				SUPPLICANT_INTF ".Interface", "disconnect");
 	if (message == NULL)
@@ -1570,7 +1576,8 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 	if (result.name != NULL && result.name[0] != '\0')
 		connman_network_set_name(network, result.name);
 
-	connman_network_set_blob(network, "WiFi.SSID",
+	if (result.ssid_len != 0)
+		connman_network_set_blob(network, "WiFi.SSID",
 						result.ssid, result.ssid_len);
 
 	connman_network_set_string(network, "WiFi.Mode", mode);
@@ -1870,11 +1877,6 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 	if (state == WPA_INVALID)
 		return;
 
-	if (task->scanning == TRUE && state != WPA_SCANNING) {
-		connman_device_cleanup_scanning(task->device);
-		task->scanning = FALSE;
-	}
-
 	prevstate = task->state;
 	task->state = state;
 
