@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -34,17 +34,16 @@
 #include "connman.h"
 
 struct connman_ipv4 {
-	enum connman_ipv4_method method;
+	enum connman_ipconfig_method method;
 	struct in_addr address;
 	struct in_addr netmask;
 	struct in_addr broadcast;
 };
 
-static int set_ipv4(struct connman_element *element,
-			struct connman_ipv4 *ipv4, const char *nameserver)
+static int set_ipv4(struct connman_element *element, struct connman_ipv4 *ipv4)
 {
 	struct ifreq ifr;
-	struct sockaddr_in *addr;
+	struct sockaddr_in addr;
 	int sk, err;
 
 	DBG("element %p ipv4 %p", element, ipv4);
@@ -63,27 +62,30 @@ static int set_ipv4(struct connman_element *element,
 
 	DBG("ifname %s", ifr.ifr_name);
 
-	addr = (struct sockaddr_in *) &ifr.ifr_addr;
-	addr->sin_family = AF_INET;
-	addr->sin_addr = ipv4->address;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr = ipv4->address;
+	memcpy(&ifr.ifr_addr, &addr, sizeof(ifr.ifr_addr));
 
 	err = ioctl(sk, SIOCSIFADDR, &ifr);
 
 	if (err < 0)
 		DBG("address setting failed (%s)", strerror(errno));
 
-	addr = (struct sockaddr_in *) &ifr.ifr_netmask;
-	addr->sin_family = AF_INET;
-	addr->sin_addr = ipv4->netmask;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr = ipv4->netmask;
+	memcpy(&ifr.ifr_netmask, &addr, sizeof(ifr.ifr_netmask));
 
 	err = ioctl(sk, SIOCSIFNETMASK, &ifr);
 
 	if (err < 0)
 		DBG("netmask setting failed (%s)", strerror(errno));
 
-	addr = (struct sockaddr_in *) &ifr.ifr_broadaddr;
-	addr->sin_family = AF_INET;
-	addr->sin_addr = ipv4->broadcast;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr = ipv4->broadcast;
+	memcpy(&ifr.ifr_broadaddr, &addr, sizeof(ifr.ifr_broadaddr));
 
 	err = ioctl(sk, SIOCSIFBRDADDR, &ifr);
 
@@ -92,18 +94,13 @@ static int set_ipv4(struct connman_element *element,
 
 	close(sk);
 
-	if (nameserver == NULL)
-		connman_error("No nameserver for %s defined", ifr.ifr_name);
-
-	connman_resolver_append(ifr.ifr_name, NULL, nameserver);
-
 	return 0;
 }
 
 static int clear_ipv4(struct connman_element *element)
 {
 	struct ifreq ifr;
-	struct sockaddr_in *addr;
+	struct sockaddr_in addr;
 	int sk, err;
 
 	DBG("element %p", element);
@@ -122,11 +119,10 @@ static int clear_ipv4(struct connman_element *element)
 
 	DBG("ifname %s", ifr.ifr_name);
 
-	connman_resolver_remove_all(ifr.ifr_name);
-
-	addr = (struct sockaddr_in *) &ifr.ifr_addr;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = INADDR_ANY;
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = INADDR_ANY;
+	memcpy(&ifr.ifr_addr, &addr, sizeof(ifr.ifr_addr));
 
 	//err = ioctl(sk, SIOCDIFADDR, &ifr);
 	err = ioctl(sk, SIOCSIFADDR, &ifr);
@@ -171,7 +167,7 @@ static int ipv4_probe(struct connman_element *element)
 	struct connman_element *connection;
 	struct connman_ipv4 ipv4;
 	const char *address = NULL, *netmask = NULL, *broadcast = NULL;
-	const char *nameserver = NULL;
+	const char *nameserver = NULL, *timeserver = NULL;
 
 	DBG("element %p name %s", element, element->name);
 
@@ -184,6 +180,8 @@ static int ipv4_probe(struct connman_element *element)
 
 	connman_element_get_value(element,
 			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_TIMESERVER, &timeserver);
 
 	DBG("address %s", address);
 	DBG("netmask %s", netmask);
@@ -195,9 +193,22 @@ static int ipv4_probe(struct connman_element *element)
 	memset(&ipv4, 0, sizeof(ipv4));
 	ipv4.address.s_addr = inet_addr(address);
 	ipv4.netmask.s_addr = inet_addr(netmask);
-	ipv4.broadcast.s_addr = inet_addr(broadcast);
+	if (broadcast)
+		ipv4.broadcast.s_addr = inet_addr(broadcast);
+	else
+		ipv4.broadcast.s_addr = ipv4.address.s_addr |
+						~ipv4.netmask.s_addr;
 
-	set_ipv4(element, &ipv4, nameserver);
+	set_ipv4(element, &ipv4);
+
+	if (nameserver != NULL) {
+		struct connman_service *service;
+
+		service = __connman_element_get_service(element);
+		__connman_service_append_nameserver(service, nameserver);
+	}
+
+	connman_timeserver_append(timeserver);
 
 	connection = connman_element_create(NULL);
 
@@ -213,8 +224,24 @@ static int ipv4_probe(struct connman_element *element)
 
 static void ipv4_remove(struct connman_element *element)
 {
+	const char *nameserver = NULL, *timeserver = NULL;
+
 	DBG("element %p name %s", element, element->name);
 
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_TIMESERVER, &timeserver);
+
+	connman_timeserver_remove(timeserver);
+
+	if (nameserver != NULL) {
+		struct connman_service *service;
+
+		service = __connman_element_get_service(element);
+		__connman_service_remove_nameserver(service, nameserver);
+	}
+
 	clear_ipv4(element);
 }
 
